local Addon = _G[...]
local MountSlot = Addon.ActionSlots:CreateKind("mount")
local SUMMON_RANDOM_FAVORITE_MOUNT_SPELL = 150544
local SUMMON_RANDOM_FAVORITE_MOUNT_ID = _G.C_MountJournal.GetMountFromSpell(SUMMON_RANDOM_FAVORITE_MOUNT_SPELL)
local GetMountInfoByID = _G.C_MountJournal.GetMountInfoByID

local function GetMountID(index)
    return select(12, _G.C_MountJournal.GetDisplayedMountInfo(index))
end

local function withSpellID(with, without)
    if type(without) == "function" then
        return function(self)
            local spellID = self:GetSpellID()

            if spellID then
                return with(spellID)
            end

            return without(self)
        end
    end

    return function(self)
        local spellID = self:GetSpellID()

        if spellID then
            return with(spellID)
        end
    end
end

MountSlot.GetActionCharges = withSpellID(
    GetSpellCharges,
    function()
        return 0, 0, 0, 0, 0
    end
)

MountSlot.GetActionCooldown = withSpellID(
    GetSpellCooldown,
    function()
        return 0, 0, 0, 0
    end
)

MountSlot.GetActionCount = withSpellID(
    GetSpellCount,
    function()
        return 0
    end
)

function MountSlot:GetActionID()
    return 0
end

function MountSlot:GetActionInfo()
    return "summonmount", self:GetAttribute("mount")
end

MountSlot.GetActionLossOfControlCooldown = withSpellID(
    GetSpellLossOfControlCooldown,
    function()
        return 0, 0
    end
)

function MountSlot:GetActionText()
    return ""
end

function MountSlot:GetActionTexture()
    local mountID = self:GetAttribute("mount")

    if mountID == SUMMON_RANDOM_FAVORITE_MOUNT_ID then
        return (select(3, _G.GetSpellInfo(SUMMON_RANDOM_FAVORITE_MOUNT_SPELL)))
    end

    return (select(3, GetMountInfoByID(mountID)))
end

function MountSlot:GetItemID()
    return 0
end

function MountSlot:GetSpellID()
    local mountID = self:GetAttribute("mount")

    if mountID == SUMMON_RANDOM_FAVORITE_MOUNT_ID then
        return SUMMON_RANDOM_FAVORITE_MOUNT_SPELL
    end

    local _, spellID = GetMountInfoByID(mountID)
    if spellID then
        return spellID
    end

    return 0
end

function MountSlot:HasAction()
    return true
end

function MountSlot:IsActionInRange()
end

function MountSlot:IsAttackAction()
    return false
end

function MountSlot:IsAutoCastAction()
    return false
end

function MountSlot:IsAutoRepeatAction()
    return false
end

function MountSlot:IsConsumableAction()
    return false
end

MountSlot.IsCurrentAction = withSpellID(IsCurrentSpell)

function MountSlot:IsEquippedAction()
    return false
end

function MountSlot:IsItemAction()
    return false
end

function MountSlot:IsStackableAction()
    return false
end

MountSlot.IsUsableAction = withSpellID(IsUsableSpell)

function MountSlot:SetTooltip(tooltip)
    local spellID = self:GetSpellID()

    if spellID then
        return tooltip:SetMountBySpellID(spellID)
    end
end

function MountSlot.Pickup(mountID)
    if mountID == SUMMON_RANDOM_FAVORITE_MOUNT_ID then
        return _G.C_MountJournal.Pickup(0)
    end

    for i = 1, _G.C_MountJournal.GetNumDisplayedMounts() do
        if GetMountID(i) == mountID then
            _G.C_MountJournal.Pickup(i)
            return true
        end
    end

    return false
end

Addon.MountSlot = MountSlot
