local AddonName = ...
local Addon = _G[AddonName]
local MovementSlot = Addon.ActionSlots:CreateKind("movement")

local MOVEMENT_SLOT_IDS = {
    "MOVEFORWARD",
    "MOVEBACKWARD",
    "TURNLEFT",
    "TURNRIGHT",
    "STRAFELEFT",
    "STRAFERIGHT",
    "JUMP"
}

local MOVEMENT_SLOT_ICONS = {
    ["MOVEFORWARD"] = [[Interface\Icons\misc_arrowlup]],
    ["MOVEBACKWARD"] = [[Interface\Icons\misc_arrowdown]],
    ["TURNLEFT"] = ([[Interface\Addons\%s\images\rotate_left.tga]]):format(AddonName),
    ["TURNRIGHT"] = ([[Interface\Addons\%s\images\rotate_right.tga]]):format(AddonName),
    ["STRAFELEFT"] = [[Interface\Icons\misc_arrowleft]],
    ["STRAFERIGHT"] = [[Interface\Icons\misc_arrowright]],
    ["JUMP"] = [[Interface\Icons\ACHIEVEMENT_GUILDPERK_FASTTRACK]]
}

local MOVEMENT_STATES = {
    ["MOVEFORWARD"] = false,
    ["MOVEBACKWARD"] = false,
    ["TURNLEFT"] = false,
    ["TURNRIGHT"] = false,
    ["STRAFELEFT"] = false,
    ["STRAFERIGHT"] = false,
    ["JUMP"] = false
}

function MovementSlot:GetActionCharges()
    return 0, 0, 0, 0, 0
end

function MovementSlot:GetActionCooldown()
    return 0, 0, 0, 0
end

function MovementSlot:GetActionCount()
    return 0
end

function MovementSlot:GetActionID()
    return 0
end

function MovementSlot:GetActionInfo()
    return
end

function MovementSlot:GetActionLossOfControlCooldown()
    return 0, 0
end

function MovementSlot:GetActionText()
    return ""
end

function MovementSlot:GetActionTexture()
    return MOVEMENT_SLOT_ICONS[self:GetAttribute("movement")]
end

function MovementSlot:GetItemID()
    return 0
end

function MovementSlot:GetSpellID()
    return 0
end

function MovementSlot:HasAction()
    return true
end

function MovementSlot:IsActionInRange()
end

function MovementSlot:IsAttackAction()
    return false
end

function MovementSlot:IsAutoCastAction()
    return false
end

function MovementSlot:IsAutoRepeatAction()
    return false
end

function MovementSlot:IsConsumableAction()
    return false
end

function MovementSlot:IsCurrentAction()
    return MOVEMENT_STATES[self:GetAttribute("movement")]
end

function MovementSlot:IsEquippedAction()
    return false
end

function MovementSlot:IsItemAction()
    return false
end

function MovementSlot:IsStackableAction()
    return false
end

function MovementSlot:IsUsableAction()
    return true
end

function MovementSlot:SetTooltip(tooltip)
    local name = _G[("BINDING_NAME_%s"):format(self:GetAttribute("movement"))]

    GameTooltip_SetTitle(tooltip, name, HIGHLIGHT_FONT_COLOR)
end

function MovementSlot.Pickup(action)
    local icon = MOVEMENT_SLOT_ICONS[action]

    return Addon.SecureEnv:PickupCustom("movement", action, icon)
end

function Addon.GetMovementSlotInfo(id)
    if id ~= nil then
        local icon = MOVEMENT_SLOT_ICONS[id]

        if icon then
            return _G[("BINDING_NAME_%s"):format(id)], icon
        end
    end
end

function Addon.GetMovementSlotIDs()
    return ipairs(MOVEMENT_SLOT_IDS)
end

for k, v in pairs{
    ["MOVEFORWARD"] = {"MoveForwardStart", "MoveForwardStop"},
    ["MOVEBACKWARD"] = {"MoveBackwardStart", "MoveBackwardStop"},
    ["TURNLEFT"] = {"TurnLeftStart", "TurnLeftStop"},
    ["TURNRIGHT"] = {"TurnRightStart", "TurnRightStop"},
    ["STRAFELEFT"] = {"StrafeLeftStart", "StrafeLeftStop"},
    ["STRAFERIGHT"] = {"StrafeRightStart", "StrafeRightStop"},
    ["JUMP"] = {"JumpOrAscendStart", "AscendStop"}
} do
    hooksecurefunc(v[1], function()
        if not MOVEMENT_STATES[k] then
            MOVEMENT_STATES[k] = true
            Addon:Notify("OnMovementStateUpdated", k, true)
        end
    end)

    hooksecurefunc(v[2], function()
        if MOVEMENT_STATES[k] then
            MOVEMENT_STATES[k] = false
            Addon:Notify("OnMovementStateUpdated", k, false)
        end
    end)
end

Addon.MovementSlot = MovementSlot