local Addon = _G[...]
local Updater = Addon:NewModule("ActionSlotUpdater", "AceEvent-3.0", "AceConsole-3.0")

local UPDATE_DELAY = 0.15

local function callMethod(target, method, ...)
    local func = target[method]

    if type(func) == "function" then
        return func(target, ...)
    end

    error(("%s does not contain a method named %q"):format(target:GetName(), method), 2)
end

function Updater:OnInitialize()
    self.active = {}
    self.usable = {}
    self.slotColors = {}

    self.rangeUpdater = CreateFrame("Frame", nil, _G.UIParent)

    self.rangeUpdater:Hide()

    self.rangeUpdater.remain = 0

    self.rangeUpdater:SetScript("OnUpdate", function(rangeUpdater, elapsed)
        local remain = rangeUpdater.remain - elapsed

        if remain > 0 then
            rangeUpdater.remain = remain
        else
            rangeUpdater.remain = UPDATE_DELAY
            if not self:UpdateRangeIndicators() then
                rangeUpdater:Hide()
            end
        end
    end)

    Addon.RegisterCallback(self, "OnActionSlotActivated")
    Addon.RegisterCallback(self, "OnActionSlotDeactivated")
    Addon.RegisterCallback(self, "OnActionSlotGainedAction")
    Addon.RegisterCallback(self, "OnActionSlotLostAction")
end

function Updater:OnEnable()
	self:RegisterEvent("ACTIONBAR_HIDEGRID")
	self:RegisterEvent("ACTIONBAR_SHOWGRID")
	self:RegisterEvent("ACTIONBAR_SLOT_CHANGED")
	self:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN")
	self:RegisterEvent("ACTIONBAR_UPDATE_STATE")
	self:RegisterEvent("ACTIONBAR_UPDATE_USABLE")
    self:RegisterEvent("COMPANION_UPDATE")
    self:RegisterEvent("CVAR_UPDATE")
	self:RegisterEvent("LOSS_OF_CONTROL_ADDED")
	self:RegisterEvent("PET_BAR_UPDATE")
	self:RegisterEvent("PET_STABLE_SHOW")
	self:RegisterEvent("PLAYER_ENTER_COMBAT")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("PLAYER_LEAVE_COMBAT")
	self:RegisterEvent("PLAYER_MOUNT_DISPLAY_CHANGED")
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	self:RegisterEvent("SPELL_ACTIVATION_OVERLAY_GLOW_HIDE")
	self:RegisterEvent("SPELL_ACTIVATION_OVERLAY_GLOW_SHOW")
	self:RegisterEvent("SPELL_UPDATE_CHARGES")
	self:RegisterEvent("SPELL_UPDATE_ICON")
	self:RegisterEvent("START_AUTOREPEAT_SPELL")
	self:RegisterEvent("STOP_AUTOREPEAT_SPELL")
	self:RegisterEvent("TRADE_SKILL_CLOSE")
	self:RegisterEvent("TRADE_SKILL_SHOW")
	self:RegisterEvent("UNIT_AURA")
	self:RegisterEvent("UNIT_ENTERED_VEHICLE")
	self:RegisterEvent("UNIT_EXITED_VEHICLE")
	self:RegisterEvent("UNIT_FLAGS")
	self:RegisterEvent("UNIT_INVENTORY_CHANGED")
	self:RegisterEvent("UPDATE_SUMMONPETS_ACTION")
    self:RegisterEvent("ACTIONBAR_PAGE_CHANGED")
    self:RegisterEvent("ARCHAEOLOGY_CLOSED")
    self:RegisterEvent("LOSS_OF_CONTROL_UPDATE")
    self:RegisterEvent("MODIFIER_STATE_CHANGED")
    self:RegisterEvent("PET_STABLE_UPDATE")
    self:RegisterEvent("UPDATE_BONUS_ACTIONBAR")
    self:RegisterEvent("UPDATE_MACROS")
    self:RegisterEvent("UPDATE_OVERRIDE_ACTIONBAR")
    self:RegisterEvent("UPDATE_POSSESS_BAR")
    self:RegisterEvent("UPDATE_SHAPESHIFT_FORM")
    self:RegisterEvent("UPDATE_VEHICLE_ACTIONBAR")

    Addon.RegisterCallback(self, "OnMovementStateUpdated")
    Addon.RegisterCallback(self, "OnFrameBindingsUpdated")
    Addon.RegisterCallback(self, "OnSettingChanged")
    Addon.RegisterCallback(self, "OnCursorPickupAction")
    Addon.RegisterCallback(self, "OnCursorDropAction")
end

function Updater:OnActionSlotActivated(msg, slot)
    self.active[slot] = true
end

function Updater:OnActionSlotDeactivated(msg, slot)
    self.active[slot] = nil
    self.usable[slot] = nil
end

function Updater:OnActionSlotGainedAction(msg, slot)
    self.usable[slot] = true
    self.rangeUpdater:Show()
end

function Updater:OnActionSlotLostAction(msg, slot)
    self.usable[slot] = nil
end

function Updater:OnCursorDropAction()
    self:ForActive("HideGrid")
end

function Updater:OnCursorPickupAction()
    self:ForActive("ShowGrid")
end

function Updater:OnFrameBindingsUpdated(msg, frame)
    if frame and self.active[frame] then
        frame:UpdateHotkey()
    end
end

function Updater:OnSettingChanged(msg, key, value)
    if key == "showEmptyActionButtons" then
        self:ForActive("UpdateGrid")
    elseif key == "showActionButtonBindings" then
        self:ForActive("UpdateHotkey")
    elseif key == "showActionButtonLabels" then
        self:ForUsable("UpdateText")
    elseif key == "showActionButtonEquippedItems" then
        self:ForUsable("UpdateEquipped")
    end
end

function Updater:OnMovementStateUpdated()
    self:ForKind("movement", "UpdateState")
end

function Updater:ACTIONBAR_HIDEGRID()
    self:ForActive("HideGrid")
end

function Updater:ACTIONBAR_SHOWGRID()
    self:ForActive("ShowGrid")
end

function Updater:ACTIONBAR_SLOT_CHANGED(event, slot)
    if slot > 0 then
        self:ForAction(slot, "Update")
    else
        self:ForActions("Update")
    end

    self:ForKind("macro", "Update")
end

function Updater:ACTIONBAR_UPDATE_COOLDOWN()
    self:ForUsable("UpdateCooldown")
end

function Updater:ACTIONBAR_UPDATE_STATE()
    self:ForUsable("UpdateCount")
    self:ForUsable("UpdateState")
    self:ForUsable("UpdatePushed")
end

function Updater:ACTIONBAR_UPDATE_USABLE()
    self:RequestRangeUpdate()
end

function Updater:ACTIONBAR_PAGE_CHANGED()
    self:ForKind("actionpage", "UpdateState")
end

function Updater:ARCHAEOLOGY_CLOSED()
    self:ForUsable("UpdateState")
end

function Updater:COMPANION_UPDATE(event, companionType, ...)
    self:ForUsable("UpdateState")
end

function Updater:CVAR_UPDATE(event, arg1, value)
    if arg1 == "LOCK_ACTIONBAR_TEXT" then
        self:ForActive("SetLocked", tonumber(value) == 1)
    end
end

function Updater:LOSS_OF_CONTROL_ADDED()
    self:ForUsable("UpdateCooldown")
end

function Updater:LOSS_OF_CONTROL_UPDATE()
    self:ForUsable("UpdateCooldown")
end

function Updater:MODIFIER_STATE_CHANGED()
    self:ForKind("macro", "Update")
end

function Updater:PET_BAR_UPDATE()
    self:ForUsable("UpdateState")
    self:ForUsable("UpdateFlash")
    self:ForUsable("UpdateAutoCast")
end

function Updater:PET_STABLE_SHOW()
    self:ForActive("Update")
end

function Updater:PET_STABLE_UPDATE()
    self:ForActive("Update")
end

function Updater:PLAYER_ENTER_COMBAT()
    self:ForUsable("UpdateFlash")
end

function Updater:PLAYER_ENTERING_WORLD()
    self:ForActive("Update")
end

function Updater:PLAYER_LEAVE_COMBAT()
    self:ForUsable("UpdateFlash")
end

function Updater:PLAYER_MOUNT_DISPLAY_CHANGED()
    self:RequestRangeUpdate()
end

function Updater:PLAYER_TARGET_CHANGED()
    self:RequestRangeUpdate()
end

function Updater:SPELL_ACTIVATION_OVERLAY_GLOW_HIDE(event, spellID)
    self:ForSpellID(spellID, "ShowOverlayGlow")
end

function Updater:SPELL_ACTIVATION_OVERLAY_GLOW_SHOW(event, spellID)
    self:ForSpellID(spellID, "HideOverlayGlow")
end

function Updater:SPELL_UPDATE_CHARGES()
    self:ForUsable("UpdateCount")
end

function Updater:SPELL_UPDATE_ICON()
    self:ForUsable("UpdateIcon")
end

function Updater:START_AUTOREPEAT_SPELL()
    self:ForUsable("UpdateFlash")
end

function Updater:STOP_AUTOREPEAT_SPELL()
    self:ForUsable("UpdateFlash")
end

function Updater:TRADE_SKILL_CLOSE()
    self:ForUsable("UpdateState")
end

function Updater:TRADE_SKILL_SHOW()
    self:ForUsable("UpdateState")
end

function Updater:UNIT_AURA(event, unit)
    if unit ~= "pet" then return end

    self:ForUsable("UpdateState")
    self:ForUsable("UpdateFlash")
end

function Updater:UNIT_ENTERED_VEHICLE(event, unit)
    if unit ~= "player" then return end

    self:ForUsable("UpdateState")
end

function Updater:UNIT_EXITED_VEHICLE(event, unit)
    if unit ~= "player" then return end

    self:ForUsable("UpdateState")
end

function Updater:UNIT_FLAGS(event, unit)
    if unit ~= "pet" then return end

    self:ForUsable("UpdateState")
    self:ForUsable("UpdateFlash")
end

function Updater:UNIT_INVENTORY_CHANGED(event, unit)
    if unit ~= "player" then return end

    self:ForActive("UpdateTooltip")
    self:ForActive("UpdateCount")
end

function Updater:UPDATE_BONUS_ACTIONBAR()
    self:ForActive("Update")
end

function Updater:UPDATE_MACROS()
    self:ForKind("macro", "Update")
end

function Updater:UPDATE_OVERRIDE_ACTIONBAR()
    self:ForActive("Update")
end

function Updater:UPDATE_POSSESS_BAR()
    self:ForActive("Update")
end

function Updater:UPDATE_SHAPESHIFT_FORM()
    self:ForUsable("UpdateIcon")
end

function Updater:UPDATE_SUMMONPETS_ACTION()
    self:ForUsable("UpdateIcon")
end

function Updater:UPDATE_VEHICLE_ACTIONBAR()
    self:ForActive("Update")
end

function Updater:ForActive(method, ...)
    if type(method) == "function" then
        for slot in pairs(self.active) do
            method(slot, ...)
        end
    elseif type(method) == "string" then
        for slot in pairs(self.active) do
            callMethod(slot, method, ...)
        end
    end
end

function Updater:ForUsable(method, ...)
    if type(method) == "function" then
        for slot in pairs(self.usable) do
            method(slot, ...)
        end
    elseif type(method) == "string" then
        for slot in pairs(self.usable) do
            callMethod(slot, method, ...)
        end
    end
end

function Updater:ForKind(kind, method, ...)
    if type(method) == "function" then
        for slot in pairs(self.usable) do
            if slot:GetAttribute("kind") == kind then
                method(slot, ...)
            end
        end
    elseif type(method) == "string" then
        for slot in pairs(self.usable) do
            if slot:GetAttribute("kind") == kind then
                callMethod(slot, method, ...)
            end
        end
    end
end

function Updater:ForAction(id, method, ...)
    if type(method) == "function" then
        for slot in pairs(self.usable) do
            if slot:GetActionID() == id then
                method(slot, ...)
            end
        end
    elseif type(method) == "string" then
        for slot in pairs(self.usable) do
            if slot:GetActionID() == id then
                callMethod(slot, method, ...)
            end
        end
    end
end

function Updater:ForSpellID(id, method, ...)
    if type(method) == "function" then
        for slot in pairs(self.usable) do
            if slot:GetSpellID() == id then
                method(slot, ...)
            end
        end
    elseif type(method) == "string" then
        for slot in pairs(self.usable) do
            if slot:GetSpellID() == id then
                callMethod(slot, method, ...)
            end
        end
    end
end

function Updater:ForActions(method, ...)
    if type(method) == "function" then
        for slot in pairs(self.usable) do
            if slot:GetAttribute("type") == "action" then
                method(slot, ...)
            end
        end
    elseif type(method) == "string" then
        for slot in pairs(self.usable) do
            if slot:GetAttribute("type") == "action" then
                callMethod(slot, method, ...)
            end
        end
    end
end

function Updater:RequestRangeUpdate()
    self.rangeUpdater.remain = 0
end

function Updater:UpdateRangeIndicators()
    if next(self.usable) then
        for slot in pairs(self.usable) do
            slot:UpdateUsable()
        end

        return true
    end

    return false
end