-- configModeUI.lua
-- The configuration mode interface
local AddonName, Addon = ...
local ParentAddonName = GetAddOnDependencies(AddonName)
local ParentAddon = _G[ParentAddonName]

local ConfigModeUI = {}

function ConfigModeUI:Init()
	local overlay = CreateFrame("Frame", nil, UIParent, "SecureHandlerStateTemplate")
	overlay:SetFrameStrata("BACKGROUND")
	overlay:Hide()
	overlay:EnableMouse(false)
	overlay:SetAllPoints(overlay:GetParent())
	self.overlay = overlay

	self:UpdateShown()
	ParentAddon.RegisterCallback(self, "OnConfigurationModeChanged")
end


function ConfigModeUI:OnBarPropertyChanged(msg, bar, property, ...)
	if not (property == "anchor" or property == "shown") then
		local frame = Addon.DragFrame:Get(bar)

		if frame then
			frame:UpdateColor()
		end
	end
end

function ConfigModeUI:OnConfigurationModeChanged(msg, show)
	if show then
		self:Show()
	else
		self:Hide()
	end
end

function ConfigModeUI:UpdateShown()
	if ParentAddon:InConfigurationMode() then
		self:Show()
	else
		self:Hide()
	end
end

function ConfigModeUI:Show()
	ParentAddon.RegisterCallback(self, "OnBarPropertyChanged")

	self:CreateDragFrames()
	self.overlay:Show()
end

function ConfigModeUI:Hide()
	ParentAddon.UnregisterCallback(self, "OnBarPropertyChanged")
	self.overlay:Hide()
end

function ConfigModeUI:CreateDragFrames()
	for _, frame in ParentAddon.Bar:GetAll() do
		Addon.DragFrame:GetOrCreate(self.overlay, frame)
	end
end

ConfigModeUI:Init()
