local AddonName, Addon = ...
local ParentAddonName = GetAddOnDependencies(AddonName)
local L = LibStub("AceLocale-3.0"):GetLocale(ParentAddonName)

local BarMenu = Addon:CreateClass('Frame')

local nextName = Addon:CreateNameGenerator('Frame')

function BarMenu:New(parent)
	local f = self:Bind(CreateFrame('Frame', nextName(), parent or _G.UIParent, "UIPanelDialogTemplate"))

	f.panels = {}
	f:EnableMouse(true)
	f:SetToplevel(true)
	f:SetMovable(true)
	f:SetClampedToScreen(true)
	f:SetFrameStrata('DIALOG')

	-- title region
	local tr = CreateFrame('Frame', nil, f)
	tr:EnableMouse(true)
	tr:RegisterForDrag('LeftButton')
	tr:SetScript('OnDragStart', function() f:StartMoving() end)
	tr:SetScript('OnDragStop', function() f:StopMovingOrSizing() end)
	tr:SetAllPoints(f:GetName() .. 'TitleBG')

	--title text
	local text = f:CreateFontString(nil, 'OVERLAY', 'GameFontHighlight')
	text:SetPoint('CENTER', tr)
	f.text = text

	-- panel selector
	local panelSelector = Addon.PanelSelector:New(f)
	panelSelector:SetPoint('TOPLEFT', tr, 'BOTTOMLEFT', 2, -6)
	panelSelector:SetPoint('TOPRIGHT', tr, 'BOTTOMRIGHT', 0, -6)
	panelSelector:SetHeight(20)
	panelSelector.OnSelect = function(_, id) f:OnShowPanel(id) end
	f.panelSelector = panelSelector

	-- panel container
	local panelContainer = CreateFrame('Frame', nil, f)
	panelContainer:SetPoint('TOPLEFT', panelSelector, 'BOTTOMLEFT', 4, -4)
	panelContainer:SetPoint('TOPRIGHT', panelSelector, 'BOTTOMRIGHT', 0, -4)
	panelContainer:SetPoint('BOTTOM', 0, 10)
	f.panelContainer = panelContainer

	f:SetSize(240 * 1.25, 320 * 1.25)
	return f
end

--tells the panel what frame we're pointed to
function BarMenu:SetOwner(owner)
	if self.panels then
		for _, panel in pairs(self.panels) do
			panel.container:SetOwner(owner)
		end
	end

	self.text:SetFormattedText(L.BarSettings, owner.name)
	self:Anchor(owner)
end

function BarMenu:Anchor(bar)
	local ratio = _G.UIParent:GetScale() / bar.frame:GetEffectiveScale()
	local x = bar.frame:GetLeft() / ratio
	local y = bar.frame:GetTop() / ratio

	self:ClearAllPoints()
	self:SetPoint('TOPRIGHT', _G.UIParent, 'BOTTOMLEFT', x, y)
end

function BarMenu:NewPanel(id)
	self.panelSelector:AddPanel(id)

	local panel = Addon.ScrollablePanel:New(self.panelContainer)
	panel:SetAllPoints(self.panelContainer)
	panel:Hide()

	self.panels[id] = panel

	return panel.container
end

function BarMenu:ShowPanel(id)
	return self.panelSelector:Select(id)
end

function BarMenu:OnShowPanel(id)
	if self.panels then
		for i, panel in pairs(self.panels) do
			if i == id then
				panel:Show()
			else
				panel:Hide()
			end
		end
	end
end

function BarMenu:AddLayoutPanel()
	local panel = self:NewPanel(L.Layout)

	panel:AddLayoutOptions()

	return panel
end

function BarMenu:AddAdvancedPanel()
	local panel = self:NewPanel(L.Advanced)

	panel:AddAdvancedOptions()

	return panel
end

Addon.BarMenu = BarMenu